package exemple.android;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
 
/**
 * <h4>Dans ce TD :<br>
 *  - lancer un sous activite<br>
 *  - logger des infos<br>
 *  - passer des infos entre activite</h4>
 * 
 * La classe EntryPoint implemente l'interface View.OnClickListener. Cela permet ainsi
 * d'enregistrer l'activite aupres des vues pour qu'elle puisse recevoir les evenements de "clic".<br><br>
 * 
 * TODO : creer un champ de saisi dans cette activite, puis transmettre le champ
 * dans l'activite 2 et lancer la recherche google avec le champ
 *
 *  @author fcamps
 *
 */
public class Activity1 extends Activity implements View.OnClickListener {
 
	/*
	 * Cette variable permettra de conserver une référence sur le bouton
	 * de l'interface
	 */
	private Button mButton;
	private EditText mEditText;
	 
	/*
	 * La redéfinition de la méthode onCreate(Bundle) permet d'effectuer des actions
	 * supplémentaires à l'Activity de base.
	 */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        /*
         * On créé l'interface graphique en utilisant le fichier entry.xml.
         * Ce dernier étant disponible comme ressource (dans /res/layout), il suffit d'utiliser
         * le fichier R.java et sa sous-classe layout.
         */
        setContentView(R.layout.main);
         mEditText = (EditText)findViewById(R.id.editText1);
        /*
         * Récupère une référence sur le bouton en utilisant son identifiant
         */
        mButton = (Button)findViewById(R.id.button1);
        /*
         * On enregistre l'activité auprès du bouton pour recevoir les évènements
         * "clic" provoqué par l'utilisateur.
         */
        mButton.setOnClickListener(this);
    }
 
    /*
     * La méthode onClick(View) provient de l'interface View.OnClickListener.
     */
	@Override
	public void onClick(View v)
	{
		if (v == mButton)
		{
			/*
			 * Nous sommes maintenant sûr que la vue ayant été cliquée est le bouton
			 * de notre interface. Il suffit donc de créer un nouvel Intent pour démarrer
			 * la seconde activité.
			 */
			Intent intent = new Intent(Activity1.this, Activity2.class);
			
			// envoie d'une info a l'activite 2
			intent.putExtra("example.android.Activity1", "info");
			
			// démarage de l'activite 2
			startActivity(intent);
			
			 // exit activity
            finish();
		}
	}
}