package com.example.fcamps.arduino1;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbAccessory;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.ParcelFileDescriptor;
import android.util.Log;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.Toast;
import android.widget.ToggleButton;

import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;



/**
 * <h4>Piloter une carte Arduino 16bits avec ADK</h4>
 * <p/>
 * En mode USB accessory, le materiel USB externe (ici la carte Arduino)
 * agit comme USB host et l'equipement Android en USB device.
 * <p/>
 * Cote Arduino :
 * <p/>
 * http://www.circuitsathome.com/arduino_usb_host_shield_projects
 * http://www.circuitsathome.com/category/mcu/arduino/usb-shield/android-adk
 * http://developer.android.com/guide/topics/connectivity/usb/accessory.html
 * http://www.embedded.com/design/connectivity/4437620/The-basics-of-USB-device-development-using-the-Android-frameworkThe-basics-of-USB-device-development-using-the-Android-framework
 *
 * @author fcamps
 */
public class MainActivity extends Activity {

    private static final String TAG = MainActivity.class.getSimpleName();

    private PendingIntent mPermissionIntent;
    // private static final String ACTION_USB_PERMISSION = "com.android.example.USB_PERMISSION";

    private static final String ACTION_USB_PERMISSION = "com.example.fcamps.arduino1.USB_PERMISSION";
    private boolean mPermissionRequestPending;

    private UsbManager mUsbManager;
    private UsbAccessory mAccessory;
    private ParcelFileDescriptor mFileDescriptor;
    private FileInputStream mInputStream;
    private FileOutputStream mOutputStream;

    private boolean state = false;

    private ToggleButton ledToggleButton;


    private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            Toast.makeText(getApplicationContext(), "detection = " + action, Toast.LENGTH_LONG).show();

            // si on recoit ACTION_USB_PERMISSION
            if (ACTION_USB_PERMISSION.equals(action)) {
                synchronized (this) {

                    UsbAccessory accessory = (UsbAccessory)intent.getParcelableExtra(UsbManager.EXTRA_ACCESSORY);

                    Toast.makeText(getApplicationContext(), "Attached", Toast.LENGTH_LONG).show();

                    if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
                        openAccessory(accessory);
                    } else {
                        Log.d(TAG, "permission denied for accessory " + accessory);

                        Toast.makeText(getApplicationContext(), "permission denied", Toast.LENGTH_LONG).show();
                    }
                    mPermissionRequestPending = false;
                }
            } else if (UsbManager.ACTION_USB_ACCESSORY_DETACHED.equals(action)) {

                UsbAccessory accessory = (UsbAccessory)intent.getParcelableExtra(UsbManager.EXTRA_ACCESSORY);

                if (accessory != null && accessory.equals(mAccessory)) {
                    closeAccessory();
                }
            }
        }
    };


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // instance sur le manager USB
        mUsbManager = (UsbManager) getSystemService(Context.USB_SERVICE);

        Context context = getApplicationContext();

        // mUsbManager = (UsbManager) context.getSystemService(Context.USB_SERVICE);


        if (mUsbManager == null) {
            //        Context context = getApplicationContext();
            CharSequence text = "Equipement non pret!";
            int duration = Toast.LENGTH_SHORT;
            Toast toast = Toast.makeText(context, text, duration);
            toast.show();
        }

        // voir http://developer.android.com/reference/android/app/PendingIntent.html
        // By giving a PendingIntent to another application, you are granting it the right*
        // to perform the operation you have specified as if the other application was yourself (with the same permissions and identity).
        mPermissionIntent = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_USB_PERMISSION), 0);

        // reception des intent USB
        IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);
        filter.addAction(UsbManager.ACTION_USB_ACCESSORY_DETACHED);

        // on enregistre le listener "mUsbReceiver" avec le filtre
        registerReceiver(mUsbReceiver, filter);

        // display
        setContentView(R.layout.main);
        ledToggleButton = (ToggleButton) findViewById(R.id.led_toggle_button);
        ledToggleButton.setOnCheckedChangeListener(toggleButtonCheckedListener);


    }

    /**
     * Called when the activity is resumed from its paused state and immediately
     * after onCreate().
     */
    @Override
    public void onResume() {
        super.onResume();

        if (mInputStream != null && mOutputStream != null) {
            return;
        }

        UsbAccessory[] accessories = mUsbManager.getAccessoryList();
        UsbAccessory accessory = null;



        if (accessories != null) {
            accessory = accessories[0];

            if (mUsbManager.hasPermission(accessory)) {
                openAccessory(accessory);
            } else {
                synchronized (mUsbReceiver) {
                    // l'USB a deja l'authorisation ?
                    if (!mPermissionRequestPending) {
                        // on donne l'authorisation au mUsbManager
                        mUsbManager.requestPermission(accessory, mPermissionIntent);
                        mPermissionRequestPending = true;
                    }
                }
            }
        } else {
            Toast.makeText(getApplicationContext(), "mAccessory is null", Toast.LENGTH_LONG).show();

            Log.d(TAG, "mAccessory is null");
        }

    }

    /**
     * Called when the activity is paused by the system.
     */
    @Override
    public void onPause() {
        super.onPause();
        closeAccessory();
    }

    /**
     * Called when the activity is no longer needed prior to being removed from
     * the activity stack.
     */
    @Override
    public void onDestroy() {
        super.onDestroy();
        unregisterReceiver(mUsbReceiver);
    }

    /**
     *
     */
    OnCheckedChangeListener toggleButtonCheckedListener = new OnCheckedChangeListener() {

        @Override
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            if (buttonView.getId() == R.id.led_toggle_button) {
                new AsyncTask<Boolean, Void, Void>() {
                    @Override
                    protected Void doInBackground(Boolean... params) {
                        blinkLED();
                        return null;
                    }
                }.execute(isChecked);
            }
        }
    };



    /**
     * @param accessory
     */
    private void openAccessory(UsbAccessory accessory) {
        mFileDescriptor = mUsbManager.openAccessory(accessory);

        if (mFileDescriptor != null) {
            mAccessory = accessory;
            FileDescriptor fd = mFileDescriptor.getFileDescriptor();
            mInputStream = new FileInputStream(fd);
            mOutputStream = new FileOutputStream(fd);
            Log.d(TAG, "accessory opened");
        } else {
            Log.d(TAG, "accessory open fail");
        }
    }

    /**
     *
     */
    private void closeAccessory() {
        try {
            if (mFileDescriptor != null) {
                mFileDescriptor.close();
            }
        } catch (IOException e) {
        } finally {
            mFileDescriptor = null;
            mAccessory = null;
        }
    }


    /**
     * Affichage de la LED
     */
    public void blinkLED() {

        byte[] buffer = new byte[1];

        if (state == false) {
            buffer[0] = (byte) 1; // button says on, light is on
            state = true;
        } else {
            buffer[0] = (byte) 0; // button says off, light is off
            state = false;
        }

        if (mOutputStream != null) {
            try {
                mOutputStream.write(buffer);
             //   Toast.makeText(getApplicationContext(), "write success", Toast.LENGTH_LONG).show();
            } catch (IOException e) {
                Log.e(TAG, "write failed", e);
            }
        }
    }

    @Override
    public void onStart() {
        super.onStart();


    }

    @Override
    public void onStop() {
        super.onStop();
    }
}


/*

// Programme C dans la carte Arduino
//
// Rajouter la lib suivante avec l'IDE Arduino
// bib USB + ADK -->
// https://github.com/felis/USB_Host_Shield_2.0
// https://github.com/felis/USB_Host_Shield_2.0/tree/master/examples

 #include <Usb.h>
 #include <adk.h>

 USB Usb;
 ADK adk(&Usb,"formation", // Manufacturer Name
 "ArduinoBlinkLED", // Model Name
 "Example sketch for the USB Host Shield", // Description (user-visible string)
 "1.0", // Version
 "http://google.com", // URL (web page to visit if no installed apps support the accessory)
 "123456789"); // Serial Number (optional)

 #define LED 13 // Pin 13 is occupied by the SCK pin on a normal Arduino (Uno, Duemilanove etc.), so use a different pin

 void setup()
 {
 Serial.begin(115200);
 Serial.print("\r\nADK demo start");
 if (Usb.Init() == -1) {
 Serial.print("\r\nOSCOKIRQ failed to assert");
 while(1); //halt
 }
 pinMode(LED, OUTPUT);
 }

 void loop()
 {
 Usb.Task();
 if(adk.isReady()) {
 uint8_t msg[1];
 uint16_t len = sizeof(msg);
 uint8_t rcode = adk.RcvData(&len, msg);
 if(rcode && rcode != hrNAK)
 USBTRACE2("Data rcv. :", rcode);
 if(len > 0) {
 Serial.print(F("\r\nData Packet: "));
 Serial.print(msg[0]);
 digitalWrite(LED,msg[0] ? HIGH : LOW);
 }
 }
 else
 digitalWrite(LED, LOW);
 }

 */