/***
 * 
 ***/
package com.firstopenglproject.android;

import static android.opengl.GLES20.GL_COLOR_BUFFER_BIT;
import static android.opengl.GLES20.GL_FLOAT;
import static android.opengl.GLES20.GL_LINES;
import static android.opengl.GLES20.GL_POINTS;
import static android.opengl.GLES20.GL_TRIANGLE_FAN;
import static android.opengl.GLES20.glClear;
import static android.opengl.GLES20.glClearColor;
import static android.opengl.GLES20.glDrawArrays;
import static android.opengl.GLES20.glEnableVertexAttribArray;
import static android.opengl.GLES20.glGetAttribLocation;
import static android.opengl.GLES20.glGetUniformLocation;
import static android.opengl.GLES20.glUniform4f;
import static android.opengl.GLES20.glUniformMatrix4fv;
import static android.opengl.GLES20.glUseProgram;
import static android.opengl.GLES20.glVertexAttribPointer;
import static android.opengl.GLES20.glViewport;
import static android.opengl.Matrix.multiplyMM;
import static android.opengl.Matrix.rotateM;
import static android.opengl.Matrix.setIdentityM;
import static android.opengl.Matrix.translateM;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.opengl.GLES20;
import android.opengl.Matrix;
import android.opengl.GLSurfaceView.Renderer;
import android.os.SystemClock;
import android.util.LoggerConfig;
import android.util.MatrixHelper;
import android.util.ShaderHelper;
import android.util.TextResourceReader;

import com.firstopenglproject.android.R;

/**
 * TODO : 
 * 
 * Ajouter l'effet de perspective
 * Ajout de l'interface interactive
 * Ajout de l'animation
 * 
 * @author fcamps
 *
 */
public class FirstOpenGLProjectRenderer implements Renderer {                       
    private static final String U_MATRIX = "u_Matrix";

    private static final String A_POSITION = "a_Position";
    private static final String A_COLOR = "a_Color";

    private static final int POSITION_COMPONENT_COUNT = 3;

    private static final int COLOR_COMPONENT_COUNT = 3;    

    private static final int BYTES_PER_FLOAT = 4;

    private static final int STRIDE = 
        (POSITION_COMPONENT_COUNT + COLOR_COMPONENT_COUNT) * BYTES_PER_FLOAT;    

    private final FloatBuffer vertexData;
    private final Context context;

    private final float[] projectionMatrix = new float[16];

    private final float[] modelMatrix = new float[16];

    final float[] tempRotat = new float[16];
    
    volatile float x=1.0f, y=0.0f, z=0.0f;
    
    int state =0;
    
    public volatile float mAngle;

    private int program;
    private int uMatrixLocation;
    private int aPositionLocation;
    private int aColorLocation;

    /**
     * Constructeur
     * 
     * @param context
     */
    public FirstOpenGLProjectRenderer(Context context) 
    {
        this.context = context;

        // Matrice de representation d'un triangle
        // Ordre des coordonnees: X, Y, Z, R, G, B
        float[] tableVerticesWithTriangles = {   

            // Triangle Fan
            -0.5f, -0.5f, 0.0f, 0.0f,  0.0f, 1.0f,        /* blue */
            0.5f,  -0.5f, 0.0f, 1.0f,  0.0f, 0.0f,        /* red */    
            0.0f,   0.5f, 0.0f, 0.0f,  1.0f, 0.0f         /* green */
        };               
        
        // Tableau des couleurs : http://lslwiki.net/lslwiki/wakka.php?wakka=color

        // Allocation de la matrice dans le code natif + ordre de lecture ENDIAN / BIG ENDIAN
        vertexData = ByteBuffer
        .allocateDirect(tableVerticesWithTriangles.length * BYTES_PER_FLOAT)
        .order(ByteOrder.nativeOrder()).asFloatBuffer();

        // Transfert de la matrice
        vertexData.put(tableVerticesWithTriangles);
    }

    @Override
    public void onSurfaceCreated(GL10 glUnused, EGLConfig config)
    {
        // Efface l'ecran
        glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

        String vertexShaderSource = TextResourceReader.readTextFileFromResource(context, R.raw.simple_vertex_shader);
        String fragmentShaderSource = TextResourceReader.readTextFileFromResource(context, R.raw.simple_fragment_shader);

        int vertexShader = ShaderHelper.compileVertexShader(vertexShaderSource);

        int fragmentShader = ShaderHelper.compileFragmentShader(fragmentShaderSource);

        program = ShaderHelper.linkProgram(vertexShader, fragmentShader);

        if (LoggerConfig.ON) {
            ShaderHelper.validateProgram(program);
        }

        glUseProgram(program);

        // On recupere l'indice de la matrice
        uMatrixLocation = glGetUniformLocation(program, U_MATRIX);

        // Index sur la position d'un vertex
        aPositionLocation = glGetAttribLocation(program, A_POSITION);

        // Index sur la couleur
        aColorLocation = glGetAttribLocation(program, A_COLOR);

        // Bind our data, specified by the variable vertexData, to the vertex
        // attribute at location A_POSITION_LOCATION.
        vertexData.position(0);
        glVertexAttribPointer(aPositionLocation, POSITION_COMPONENT_COUNT, GL_FLOAT, false, STRIDE, vertexData);
        glEnableVertexAttribArray(aPositionLocation);     

        // Bind our data, specified by the variable vertexData, to the vertex
        // attribute at location A_COLOR_LOCATION.
        vertexData.position(COLOR_COMPONENT_COUNT);   
        glVertexAttribPointer(aColorLocation, COLOR_COMPONENT_COUNT, GL_FLOAT, false, STRIDE, vertexData);        
        glEnableVertexAttribArray(aColorLocation);
    }

    /**
     * onSurfaceChanged is called whenever the surface has changed. This is
     * called at least once when the surface is initialized. Keep in mind that
     * Android normally restarts an Activity on rotation, and in that case, the
     * renderer will be destroyed and a new one created.
     * 
     * @param width
     *            The new width, in pixels.
     * @param height
     *            The new height, in pixels.
     */
    @Override
    public void onSurfaceChanged(GL10 glUnused, int width, int height) {

        // Set the OpenGL viewport to fill the entire surface.
        glViewport(0, 0, width, height);

        //public static void perspectiveM (float[] m, int offset, float fovy, float aspect, float zNear, float zFar)
        MatrixHelper.perspectiveM(projectionMatrix, 45, (float) width/ (float) height, 1f, 10f);

        setIdentityM(modelMatrix, 0);

        translateM(modelMatrix, 0, 0f, 0f, -2.5f);        
        rotateM(modelMatrix, 0, -60f, 1f, 0f, 0f);

        final float[] temp = new float[16];
        multiplyMM(temp, 0, projectionMatrix, 0, modelMatrix, 0);        
        System.arraycopy(temp, 0, projectionMatrix, 0, temp.length);
    }
    
    /**
     *  Changement de la rotation
     */
    void change()
    {
        if(state==0)
        {
            x=0.0f; y=1.0f; z=0.0f;
            state=1;
        }
        if(state==1)
        {
            x=0.0f; y=0.0f; z=1.0f;
            state=2;
        }
        if(state==2)
        {
            x=1.0f; y=0.0f; z=1.0f;
            state=0;
        }
    }
    

    /**
     * OnDrawFrame is called whenever a new frame needs to be drawn. Normally,
     * this is done at the refresh rate of the screen.
     */
    @Override
    public void onDrawFrame(GL10 glUnused) {

        // Clear the rendering surface.
        glClear(GL_COLOR_BUFFER_BIT);

        long time = SystemClock.uptimeMillis() % 20L; // reste de la division par 20
        float angle = 0.090f * ((int) time);

        // Utilisation d'une matrice entite
        setIdentityM(modelMatrix, 0);

        Matrix.setRotateM(modelMatrix, 0, angle, x, y, z);
        // printVector(modelMatrix);

        // multiplcation modelMatrix * Matrice = tempRotat (-> matrice resultat de la rotation
        // qu'il faut afficher)
        Matrix.multiplyMM(tempRotat, 0, projectionMatrix, 0, modelMatrix, 0);

        // Recopie de la matrice tempRotat dans projectionMatrix
        System.arraycopy(tempRotat, 0, projectionMatrix, 0, tempRotat.length);

        // On fait "pointer" la matrix projectionMatrix --> uMatrixLocation
        glUniformMatrix4fv(uMatrixLocation, 1, false, projectionMatrix, 0);

        // Affichage du triangle
        GLES20.glDrawArrays(GLES20.GL_TRIANGLES, 0, 3);        
    }

    /**
     * Print vector
     * 
     * @param mat
     */
    private void printVector(float[] mat)
    {
        System.out.println("**** start mat ****");

        for(int i=0; i<mat.length;i++)
            System.out.print(mat[i] + " ");

        System.out.println("**** end mat ****");
    }
}