package ioio.basics.ioio;

import android.os.Bundle;

import ioio.lib.api.AnalogInput;
import ioio.lib.api.DigitalInput;
import ioio.lib.api.DigitalOutput;
import ioio.lib.api.exception.ConnectionLostException;
import ioio.lib.util.BaseIOIOLooper;
import ioio.lib.util.IOIOLooper;
import ioio.lib.util.android.IOIOActivity;

import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.ToggleButton;


public class MainActivity extends IOIOActivity {

    private static final int PLUS_PIN = 44;
    private static final int GND_PIN = 46;
    private static final int INPUT_PIN = 45;
    private static final int ALARM = 43;

    private TextView temperature_;
    private RadioButton radioF_;
    private ToggleButton ToggleButtonAlarm;

    /**
     * Called when the activity is first created. Here we normally initialize
     * our GUI.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        temperature_ = (TextView)findViewById(R.id.temp);
        radioF_ = (RadioButton)findViewById(R.id.radio_f);
        radioF_ = (RadioButton)findViewById(R.id.radio_f);
        ToggleButtonAlarm = (ToggleButton)findViewById(R.id.ToggleButtonAlarm);
    }

    /**
     * This is the thread on which all the IOIO activity happens. It will be run
     * every time the application is resumed and aborted when it is paused. The
     * method setup() will be called right after a connection with the IOIO has
     * been established (which might happen several times!). Then, loop() will
     * be called repetitively until the IOIO gets disconnected.
     */
    class Looper extends BaseIOIOLooper
    {
        private DigitalOutput plusPin_;
        private DigitalOutput gndPin_;
        private AnalogInput inputPin_;

        private DigitalOutput led;

        @Override
        protected void setup() throws ConnectionLostException
        {
            led = ioio_.openDigitalOutput(ALARM, true);
            gndPin_ = ioio_.openDigitalOutput(GND_PIN, DigitalOutput.Spec.Mode.NORMAL, false); // gnd supply to temp sensor
            plusPin_ = ioio_.openDigitalOutput(PLUS_PIN, DigitalOutput.Spec.Mode.NORMAL, true); // positive supply to temp sensor
            inputPin_ = ioio_.openAnalogInput(INPUT_PIN);
        }

        /**
         * Called repetitively while the IOIO is connected.
         *
         * @throws ConnectionLostException
         *             When IOIO connection is lost.
         *
         */
        @Override
        public void loop() throws ConnectionLostException {

            try {
                float v = inputPin_.getVoltage();

                String units = "C";

                float temp = (v - 0.5f) * 100.0f;

                if (radioF_.isChecked())
                {
                    temp = temp * 9.0f / 5.0f + 32.0f;
                    units = "F";
                }

                if (ToggleButtonAlarm.isChecked()) {
                    led.write(true);
                }
                else {
                    led.write(false);
                }

                // round to 1 dp
                temp = Math.round(temp * 10) / 10.0f;
                updateTempField(temp, units);
                long now = System.currentTimeMillis();

                Thread.sleep(500);
            }
            catch (Exception e)
            {
                //Toast(e.getMessage());
            }
        }
    }

    /**
     * A method to create our IOIO thread.
     *
     *
     */
    @Override
    protected IOIOLooper createIOIOLooper() {
        return new Looper();
    }


    private void updateTempField(final float temp, final String units) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                temperature_.setText("" + temp + " " + units);
            }
        });
    }

}
